@@ -20,16 +20,16 @@ static const G@Type@Value @enum_name@_values[] = {
 GType
 @enum_name@_get_type (void)
 {
-    static volatile gsize g_define_type_id__volatile = 0;
+    static gsize g_define_type_id_initialized = 0;
 
-    if (g_once_init_enter (&g_define_type_id__volatile)) {
+    if (g_once_init_enter (&g_define_type_id_initialized)) {
         GType g_define_type_id =
             g_@type@_register_static (g_intern_static_string ("@EnumName@"),
                                       @enum_name@_values);
-        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+        g_once_init_leave (&g_define_type_id_initialized, g_define_type_id);
     }
 
-    return g_define_type_id__volatile;
+    return g_define_type_id_initialized;
 }
 
 /* Enum-specific method to get the value as a string.
